#include <string.h>

#include "uuutil.h"

//////////////////////////////////////////////////////////////////////////
static char uuencode_char(char c)
{
    if (c==0)
        return 0x60;
    return c+0x20;
}

//////////////////////////////////////////////////////////////////////////
static char uudecode_char(char c)
{
    if (c==0x60)
        return 0;
    return c-0x20;
}

//////////////////////////////////////////////////////////////////////////
void uuencode(char *in, int inlen, char *out, int *outlen)
{
    int outpos = 0;

    out[outpos++] = uuencode_char(inlen);

    for (int i = 0; i < inlen; i+=3)
    {
        int a = in[i];
        int b = (i+1 < inlen) ? in[i+1] : 0;
        int c = (i+2 < inlen) ? in[i+2] : 0;

        int w = ((a >> 2) & 0x3f);
        int x = (((a << 4) | ((b >> 4) & 0xf)) & 0x3f);
        int y = (((b << 2) | ((c >> 6) & 0x3)) & 0x3f);
        int z = ((c) & 0x3f);

        out[outpos++] = uuencode_char(w);
        out[outpos++] = uuencode_char(x);
        out[outpos++] = uuencode_char(y);
        out[outpos++] = uuencode_char(z);
    }

    out[outpos] = 0;

    *outlen = outpos;
}

//////////////////////////////////////////////////////////////////////////
void uudecode(char *in, char *out, int *outlen)
{
    int outpos = 0;
    int outlen_guess = uudecode_char(in[0]);
    int inlen = strlen(in);

    for (int i = 1; i < inlen; i+=4)
    {
        int w = uudecode_char(in[i+0]), 
            x = uudecode_char(in[i+1]),
            y = uudecode_char(in[i+2]),
            z = uudecode_char(in[i+3]);

        
        int a = (w<<2) | (x >> 4);
        int b = ((x<<4)&0xff) | (y >> 2);
        int c = ((y<<6)&0xff) | (z);

        out[outpos++] = a;
        out[outpos++] = b;
        out[outpos++] = c;
    }

    *outlen = outlen_guess;
}

