#include <stdlib.h>
#include <assert.h>

#include "vector.h"

vector_t *vector_create(int capacity)
{
	vector_t *v = (vector_t*) calloc(1, sizeof(vector_t));
	v->items = (void**) calloc(capacity, sizeof(void*));
	v->capacity = capacity;

	return v;
}

void vector_destroy(vector_t *v)
{
	free(v->items);
	free(v);
}

void vector_grow(vector_t *v)
{
	if (v->size < v->capacity)
		return;

	vector_ensure_capacity(v, v->size * 2);
}

void vector_ensure_capacity(vector_t *v, int newcapacity)
{
	v->items = (void**) realloc(v->items, newcapacity * sizeof(void*));
	v->capacity = newcapacity;
}

void vector_add(vector_t *v, void *item)
{
	vector_grow(v);

	v->items[v->size++] = item;
}

void vector_put(vector_t *v, int idx, void *item)
{
	assert(idx < v->size);

	v->items[idx] = item;
}

void *vector_get(vector_t *v, int idx)
{
	assert(idx < v->size);
	
	return v->items[idx];
}

int vector_size(vector_t *v)
{
	return v->size;
}

void vector_sort(vector_t *v, int (*compar)(const void *, const void *))
{
	qsort(v->items, v->size, sizeof(void*), compar);
}

void vector_delete_find(vector_t *v, void *n)
{
	int i;
	for (i = 0; i < v->size; i++) {
		if (v->items[i]==n) {
			vector_delete(v, i);
			i--;
		}
	}
}

void vector_delete(vector_t *v, int idx)
{
	int i;
	assert(idx < v->size);

	for (i = idx+1; i < v->size; i++)
		v->items[i-1] = v->items[i];

	v->size--;
}

void vector_delete_shuffle(vector_t *v, int idx)
{
	assert(idx < v->size);

	v->items[idx] = v->items[v->size-1];

	v->size--;
}

int vector_is_member(vector_t *v, void *item)
{
	int i;

	for (i = 0; i < v->size; i++)
		if (v->items[i] == item)
			return 1;

	return 0;
}

vector_iterator_t *vector_iterator_create(vector_t *v)
{
	vector_iterator_t *vi = (vector_iterator_t*) calloc(1, sizeof(vector_iterator_t));
	vi->v = v;
	vi->nextidx = 0;

	return vi;
}

void vector_iterator_destroy(vector_iterator_t *vi)
{
	free(vi);
}

void *vector_iterator_next(vector_iterator_t *vi)
{
	if (vi->nextidx >= vi->v->size)
		return NULL;

	return vi->v->items[vi->nextidx++];
}
